// %flair:license{
// This file is part of the Flair framework distributed under the
// CECILL-C License, Version 1.0.
// %flair:license}
//  created:    2016/02/08
//  filename:   SimuX4.cpp
//
//  author:     Guillaume Sanahuja
//              Copyright Heudiasyc UMR UTC/CNRS 7253
//
//  version:    $Id: $
//
//  purpose:    Class defining a simulation x4 uav
//
//
/*********************************************************************/

#include "SimuX4.h"
#include <FrameworkManager.h>
#include <X4X8Multiplex.h>
#include <SimuImu.h>
#include <SimuAhrs.h>
#include <SimuBldc.h>
#include <SimuUs.h>
#include <SimuCamera.h>
#include <BatteryMonitor.h>
#include <Tab.h>
#include <FindArgument.h>

using std::string;
using namespace flair::core;
using namespace flair::gui;
using namespace flair::sensor;
using namespace flair::filter;
using namespace flair::actuator;

namespace flair {
namespace meta {

SimuX4::SimuX4(string name, uint32_t simu_id,string options,
               filter::UavMultiplex *multiplex)
    : Uav(name, multiplex) {

  if (multiplex == NULL)
    SetMultiplex(new X4X8Multiplex("motors", X4X8Multiplex::X4));

  SetBldc(new SimuBldc(GetUavMultiplex(), GetUavMultiplex()->GetLayout(),
                       "motors", GetUavMultiplex()->MotorsCount(), simu_id,0));
  SetUsRangeFinder(new SimuUs("us", simu_id,0, 60));
  SetAhrs(new SimuAhrs("imu", simu_id, 0,70));
  Tab *bat_tab = new Tab(getFrameworkManager()->GetTabWidget(), "battery");
  SetBatteryMonitor(new BatteryMonitor(bat_tab->NewRow(), "battery"));
  GetBatteryMonitor()->SetBatteryValue(12);
  
  string camvOpts=FindArgument(options,"camv=",false);
  uint16_t camvWidth=320,camvHeight=240;
	if(camvOpts=="") {
    Info("using default vertical camera resolution: %ix%i\n",camvWidth, camvHeight);
	}
  
  SetVerticalCamera(new SimuCamera("simu_cam_v", camvWidth, camvHeight, 3, simu_id,0, 10));
  //SetHorizontalCamera(new SimuCamera("simu_cam_h", 320, 240, 3, simu_id,1, 10));
}

SimuX4::~SimuX4() {}

void SimuX4::StartSensors(void) {
  ((SimuImu *)(GetAhrs()->GetImu()))->Start();
  ((SimuUs *)GetUsRangeFinder())->Start();
  ((SimuCamera *)GetVerticalCamera())->Start();
}


} // end namespace meta
} // end namespace flair
